==> argrelay-0.7.12.dev5.tar.gz: argrelay-0.7.12.dev5/docs/task_refs/TODO_39_25_11_76.data_envelopes_with_missing_props.md <==

TODO: TODO_39_25_11_76: `data_envelope`-s with missing props

TODO: Move it to `feature_story`?

# Scenario

*   Multiple `data_envelope`-s loaded under the same `ReservedPropName.envelope_class`.
*   The `search_control` used to query them contains `some_prop_x` prop.
*   This `some_prop_x` exists in some `data_envelope`-s and does not exist in others.

# Original issue occurrence: data loaders

Those `data_envelope`-s which have `some_prop_x` will hide those which do not while narrowing down the search.

Why?

*   Function `EnvelopeContainer.populate_implicit_arg_values` will assign singled out value of `some_prop_x`.

    Note that `None`/`null` or blank values are not included (FS_99_81_19_25: no space in options).

*   The next query will not return `data_envelope`-s which do not have `some_prop_x`.

This is currently considered to be a data issue (not easily fix-able in `argrelay`)
which can be easily prevented by loaders by always assigning some `UNKNOWN` value.

# Another issue occurrence: func loader via delegators (framework logic)

For example, consider this FS_26_43_73_72 func tree:

```
"goto":
    "repo": func_id_1
    "host": func_id_2
    "sub":
        "commit": func_id_3
        "proc": func_id_4
"list":
    "repo": func_id_5
    "host": func_id_6
```

When loaded, there will be two func envelopes with props to select funcs at 3rd sub-tree level:

```
goto sub commit
goto sub proc
```

But the rest of funcs will only have props to select funcs at 2nd sub-tree level:

```
goto repo
goto host
list repo
list host
```

Because of blank prop values for 3rd sub-tree, subsequent query (after narrowing down prop values)
will not show the funcs with blank 3rd sub-tree prop values.

==> argrelay-0.7.21.tar.gz: argrelay-0.7.21/docs/task_refs/TODO_39_25_11_76.data_envelopes_with_missing_props.md <==

TODO: TODO_39_25_11_76: `data_envelope`-s with missing props

TODO: Move it to `feature_story`?

# Scenario

*   Multiple `data_envelope`-s loaded under the same `ReservedPropName.envelope_class`.
*   The `search_control` used to query them contains `some_prop_x` prop.
*   This `some_prop_x` exists in some `data_envelope`-s and does not exist in others.

# Case A: original issue occurrence: data loaders

While narrowing down the search, those `data_envelope`-s which have `some_prop_x` will hide those which do not.

Why?

*   Function `EnvelopeContainer.populate_implicit_arg_values` will assign singled out value of `some_prop_x`.

    Note that `None`/`null` or blank values are not included (FS_99_81_19_25: no space in options).

*   The next query will not return `data_envelope`-s which do not have `some_prop_x`.

Case A is currently considered to be a data issue (not easily fix-able in `argrelay`)
which can be easily prevented by loaders by always assigning some `UNKNOWN` value.

# Case B: Another issue occurrence: func loader via delegators (framework logic)

For example, consider this FS_26_43_73_72 func tree:

```
"goto":
    "repo": func_id_1
    "host": func_id_2
    "sub":
        "commit": func_id_3
        "proc": func_id_4
"list":
    "repo": func_id_5
    "host": func_id_6
```

When loaded, there will be two func envelopes with props to select funcs at 3rd sub-tree level:

```
goto sub commit
goto sub proc
```

But the rest of funcs will only have props to select funcs at 2nd sub-tree level:

```
goto repo
goto host
list repo
list host
```

Because of blank prop values for 3rd sub-tree, subsequent query (after narrowing down prop values)
will not show the funcs with blank 3rd sub-tree prop values.

Case B is solvable within the `argrelay` framework by populating surrogate values to missing func `data_envelope`-s.
